/**
 * data-structure-typed
 *
 * @author Tyler Zeng
 * @copyright Copyright (c) 2022 Tyler Zeng <zrwusa@gmail.com>
 * @license MIT License
 */
import {PriorityQueue} from './priority-queue';
import type {PriorityQueueOptions} from '../types';

export class MinPriorityQueue<T = number> extends PriorityQueue<T> {
    /**
     * The constructor initializes a PriorityQueue with optional nodes and a comparator function.
     * @param [options] - An optional object that contains the following properties:
     */
    constructor(options?: PriorityQueueOptions<T>) {
        super({
            nodes: options?.nodes, comparator: options?.comparator ? options.comparator : (a: T, b: T) => {
                const aKey = a as unknown as number, bKey = b as unknown as number;
                return aKey - bKey;
            }
        });
    }
}